@@ -16,7 +16,9 @@ otel_srcref_attributes <- function(srcref, fn_name = NULL) {
   # Semantic conventions for code: https://opentelemetry.io/docs/specs/semconv/registry/attributes/code/
   #
   # Inspiration from https://github.com/r-lib/testthat/pull/2087/files#diff-92de3306849d93d6f7e76c5aaa1b0c037e2d716f72848f8a1c70536e0c8a1564R123-R124
-  filename <- attr(srcref, "srcfile")$filename
+  srcfile <- attr(srcref, "srcfile")
+  # Prefer the original filename (as user typed it) over the normalized path
+  filename <- getSrcfileFilename(srcfile)
   dropNulls(list(
     "code.function.name" = fn_name,
     # Location attrs
